@@ -1,15 +1,35 @@
 # -*- coding: utf-8 -*-
 from django.core.exceptions import ImproperlyConfigured
-from django.db.models.fields import CharField, TextField
-from django.db.models.fields.files import FileField, ImageField
+from django.db.models import fields
 
 from modeltranslation import settings as mt_settings
-from modeltranslation.utils import (get_language,
-                                    build_localized_fieldname,
-                                    build_localized_verbose_name)
+from modeltranslation.utils import (
+    get_language, build_localized_fieldname, build_localized_verbose_name, resolution_order)
 
 
-SUPPORTED_FIELDS = (CharField, TextField, FileField, ImageField,)
+SUPPORTED_FIELDS = (
+    fields.CharField,
+    # Above implies also CommaSeparatedIntegerField, EmailField, FilePathField, SlugField
+    # and URLField as they are subclasses of CharField.
+    fields.TextField,
+    fields.IntegerField,
+    # Above implies also BigIntegerField, SmallIntegerField, PositiveIntegerField and
+    # PositiveSmallIntegerField, as they are subclasses of IntegerField.
+    fields.BooleanField,
+    fields.NullBooleanField,
+    fields.FloatField,
+    fields.DecimalField,
+    fields.IPAddressField,
+    fields.DateField,
+    fields.DateTimeField,
+    fields.TimeField,
+    fields.files.FileField,
+    fields.files.ImageField,
+)
+try:
+    SUPPORTED_FIELDS += (fields.GenericIPAddressField,)  # Django 1.4+ only
+except AttributeError:
+    pass
 
 
 def create_translation_field(model, field_name, lang):
@@ -27,8 +47,7 @@ def create_translation_field(model, field_name, lang):
     """
     field = model._meta.get_field(field_name)
     cls_name = field.__class__.__name__
-    if not (isinstance(field, SUPPORTED_FIELDS) or
-            cls_name in mt_settings.CUSTOM_FIELDS):
+    if not (isinstance(field, SUPPORTED_FIELDS) or cls_name in mt_settings.CUSTOM_FIELDS):
         raise ImproperlyConfigured(
             '%s is not supported by modeltranslation.' % cls_name)
     translation_class = field_factory(field.__class__)
@@ -38,6 +57,10 @@ def create_translation_field(model, field_name, lang):
 def field_factory(baseclass):
     class TranslationFieldSpecific(TranslationField, baseclass):
         pass
+
+    # Reflect baseclass name of returned subclass
+    TranslationFieldSpecific.__name__ = 'Translation%s' % baseclass.__name__
+
     return TranslationFieldSpecific
 
 
@@ -63,53 +86,33 @@ class TranslationField(object):
         # Update the dict of this field with the content of the original one
         # This might be a bit radical?! Seems to work though...
         self.__dict__.update(translated_field.__dict__)
-        self._post_init(translated_field, language)
 
-    def _post_init(self, translated_field, language):
-        """
-        Common init for subclasses of TranslationField.
-        """
         # Store the originally wrapped field for later
         self.translated_field = translated_field
         self.language = language
 
         # Translation are always optional (for now - maybe add some parameters
         # to the translation options for configuring this)
-        self.null = True
+
+        if not isinstance(self, fields.BooleanField):
+            # TODO: Do we really want to enforce null *at all*? Shouldn't this
+            # better honour the null setting of the translated field?
+            self.null = True
         self.blank = True
 
         # Adjust the name of this field to reflect the language
-        self.attname = build_localized_fieldname(self.translated_field.name,
-                                                 self.language)
+        self.attname = build_localized_fieldname(self.translated_field.name, self.language)
         self.name = self.attname
 
         # Copy the verbose name and append a language suffix
         # (will show up e.g. in the admin).
-        self.verbose_name = build_localized_verbose_name(
-            translated_field.verbose_name, language)
+        self.verbose_name = build_localized_verbose_name(translated_field.verbose_name, language)
 
-    def pre_save(self, model_instance, add):
-        val = self.translated_field.__class__.pre_save(
-            self, model_instance, add)
-        if mt_settings.DEFAULT_LANGUAGE == self.language and not add:
-            # Rule is: 3. Assigning a value to a translation field of the
-            # default language also updates the original field
-            model_instance.__dict__[self.translated_field.attname] = val
-        return val
-
-    def get_prep_value(self, value):
-        if value == '':
-            value = None
-        return self.translated_field.get_prep_value(value)
-
-    def get_prep_lookup(self, lookup_type, value):
-        return self.translated_field.get_prep_lookup(lookup_type, value)
-
-    def to_python(self, value):
-        return self.translated_field.to_python(value)
-
-    def get_internal_type(self):
-        return self.translated_field.get_internal_type()
+    def get_attname_column(self):
+        attname = self.get_attname()
+        column = build_localized_fieldname(
+            self.translated_field.db_column or self.translated_field.name, self.language) or attname
+        return attname, column
 
     def south_field_triple(self):
         """
@@ -123,56 +126,37 @@ class TranslationField(object):
         # That's our definition!
         return (field_class, args, kwargs)
 
-    def formfield(self, *args, **kwargs):
-        """
-        Preserves the widget of the translated field.
-        """
-        trans_formfield = self.translated_field.formfield(*args, **kwargs)
-        defaults = {'widget': type(trans_formfield.widget)}
-        defaults.update(kwargs)
-        return super(TranslationField, self).formfield(*args, **defaults)
-
 
 class TranslationFieldDescriptor(object):
     """
     A descriptor used for the original translated field.
     """
-    def __init__(self, name, initial_val='', fallback_value=None):
+    def __init__(self, field, fallback_value=None, fallback_languages=None):
         """
         The ``name`` is the name of the field (which is not available in the
         descriptor by default - this is Python behaviour).
         """
-        self.name = name
-        self.val = initial_val
+        self.field = field
         self.fallback_value = fallback_value
+        self.fallback_languages = fallback_languages
 
     def __set__(self, instance, value):
         lang = get_language()
-        loc_field_name = build_localized_fieldname(self.name, lang)
+        loc_field_name = build_localized_fieldname(self.field.name, lang)
         # also update the translation field of the current language
         setattr(instance, loc_field_name, value)
-        # update the original field via the __dict__ to prevent calling the
-        # descriptor
-        instance.__dict__[self.name] = value
 
     def __get__(self, instance, owner):
-        if not instance:
-            raise ValueError(
-                "Translation field '%s' can only be accessed via an instance "
-                "not via a class." % self.name)
-        loc_field_name = build_localized_fieldname(
-            self.name, get_language())
-        if hasattr(instance, loc_field_name):
-            if getattr(instance, loc_field_name):
-                return getattr(instance, loc_field_name)
-            elif self.fallback_value is None:
-                return self.get_default_instance(instance)
-            else:
-                return self.fallback_value
-
-    def get_default_instance(self, instance):
-        """
-        Returns default instance of the field. Supposed to be overidden by
-        related subclasses.
-        """
-        return instance.__dict__[self.name]
+        if instance is None:
+            return self
+        langs = resolution_order(get_language(), self.fallback_languages)
+        for lang in langs:
+            loc_field_name = build_localized_fieldname(self.field.name, lang)
+            val = getattr(instance, loc_field_name, None)
+            # Here we check only for None and '', because e.g. 0 should not fall back.
+            if val is not None and val != '':
+                return val
+        if self.fallback_value is None:
+            return self.field.get_default()
+        else:
+            return self.fallback_value
